      SUBROUTINE HZHRWXML(IFLAG)
c
c     Write out HERWIG parameters in XML format
c
c     IFLAG = 0 :  Write XML header
c             1 :  Open root element
c            -1 :  Close root element
c             2 :  Write reaction element
c             3 :  Open runSeries element
c            -3 :  Close runSeries element
c             4 :  Write run element (may be called several times)
c
c     Write out HERWIG parameters in XML format
c
c     Call HZXML before and after this to do generator-independent bits
c
cc      IMPLICIT NONE included in herwig65.inc
cc      IMPLICIT  NONE

#include "herwig65.inc"
#include "jimmy.inc"

      INTEGER IFLAG,IUNIT
      PARAMETER (IUNIT=12)

c     File stem for output files
      CHARACTER*80 stem

c     Buffer for formatting numbers in output
      CHARACTER*14 buff
      INTEGER buflen
      PARAMETER (buflen=14)
      
c     Function to find number of occupied characters in a string (KERNLIB)
      INTEGER lenocc

c     Lower-case versions of beam particle names
      CHARACTER*8 part1s, part2s

c     Name of PDF author group
      CHARACTER*20 PDFAUTS

      character*5 versn
      common /hwvrsn/ versn

c     Convert beam particle names to lower case: OK for p, pbar, e+, e-
      part1s = part1
      part2s = part2
      CALL CUTOL(part1s)
      CALL CUTOL(part2s)

      if (IFLAG.eq.0) then
c     File stem is first command-line argument
         call GETARG(1,stem)
c     Open file for XML output
         open(unit=iunit,file=stem(1:LENOCC(stem))//'.xml',status='new')
c     Need to do this with a format statement, otherwise you get a
c     leading space which makes JDOM barf (!)
         write(iunit,1000) 
 1000    FORMAT('<?xml version="1.0" encoding="ISO-8859-1" ?>')

      elseif (IFLAG.eq.1) then
         write (iunit,*) '<hep:hepml'//
     &     ' xsi:schemaLocation='//
     &     '"http://www.cedar.ac.uk/hepml/0.1/ herwig.xsd"'//
     &     ' xmlns:hep="http://www.cedar.ac.uk/hepml/0.1/"'//
     &     ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">'
      elseif (IFLAG.eq.-1) then
         write (iunit,*) '</hep:hepml>'
         close(unit=iunit)

      elseif (IFLAG.eq.2) then
         write (iunit,*) ' <hep:reaction>'
         write (iunit,*) '  <hep:collision>'

         write (iunit,*) '   <hep:particle name="'//
     &        PART1S(1:LENOCC(PART1S))//'" id="2">'
         write (buff,'(F14.4)') PBEAM1
         call cleft(buff,1,buflen)
         write (iunit,*) '    <hep:momentum unit="GeV">'//
     &        buff(1:lenocc(buff))//'</hep:momentum>'
         write (iunit,*) '   </hep:particle>'

         write (iunit,*) '   <hep:particle name="'//
     &        PART2S(1:LENOCC(PART2S))//'" id="1">'
         write (buff,'(F14.4)') PBEAM2
         call cleft(buff,1,buflen)
         write (iunit,*) '    <hep:momentum unit="GeV">'//
     &        buff(1:lenocc(buff))//'</hep:momentum>'
         write (iunit,*) '   </hep:particle>'

         write (iunit,*) '  </hep:collision>'
         write (iunit,*) ' </hep:reaction>'

      elseif (IFLAG.eq.3) then
         write (iunit,*) ' <hep:runSeries>'
      elseif (IFLAG.eq.-3) then
         write (iunit,*) ' </hep:runSeries>'

      elseif (IFLAG.eq.4) then
         write (iunit,*) '  <hep:run>'

         write (iunit,*) '   <hep:randomSeed>'
         write (buff,'(I14)') nrn(1)
         call cleft(buff,1,buflen)
         write (iunit,*) '    <hep:seed index="1">'//
     &        buff(1:lenocc(buff))//'</hep:seed>'
         write (buff,'(I14)') nrn(2)
         call cleft(buff,1,buflen)
         write (iunit,*) '    <hep:seed index="2">'//
     &        buff(1:lenocc(buff))//'</hep:seed>'
         write (iunit,*) '   </hep:randomSeed>'

         write (iunit,*) '   <hep:normalisation>'

         write (buff,'(E14.7)') 1000.*AVWGT
         call cleft(buff,1,buflen)
         write (iunit,*) '    <hep:crossSection unit="pb">'//
     &        buff(1:lenocc(buff))//'</hep:crossSection>'

         write (buff,'(I14)') MAXEV
         call cleft(buff,1,buflen)
         write (iunit,*) '    <hep:numberOfEvents>',
     &        buff(1:lenocc(buff))//'</hep:numberOfEvents>'

         write (buff,'(E14.7)') MAXEV/(1000.*AVWGT)
         call cleft(buff,1,buflen)
         write (iunit,*) '    <hep:luminosity unit="pb^-1">'//
     &        buff(1:lenocc(buff))//'</hep:luminosity>'

         write (iunit,*) '   </hep:normalisation>'

         write (iunit,*) '   <hep:generatorSettings>'

         write (iunit,*)
     &        '    <hep:generator name="HERWIG" version="',versn,'"/>'

         PDFAUTS = AUTPDF(1)
         write(buff, *)MODPDF(1)

         write (iunit,99) 
     &        PDFAUTS(1:LENOCC(PDFAUTS)),buff(2:LENOCC(buff))

         write (iunit,*) '     <hep:particle name="'//
     &        PART1S(1:LENOCC(PART1S))//'"/>'
         write (iunit,*) '    </hep:pdf>'
c     Only write second PDF if beam particles are different
         if (PART1S.NE.PART2S) then
            PDFAUTS = AUTPDF(2)

            write(buff, *)MODPDF(2)

            write (iunit,99) 
     &           PDFAUTS(1:LENOCC(PDFAUTS)),buff(2:LENOCC(buff))
            write (iunit,*) '     <hep:particle name="'//
     &           PART2S(1:LENOCC(PART2S))//'"/>'
            write (iunit,*) '    </hep:pdf>'
         endif
 99      FORMAT('    <hep:pdf set="',A, '" member="',
     &        A, '" scheme="LHAPDF">')

         call hzxmld(iunit,'CLMAX',CLMAX)
         call hzxmld(iunit, 'CLPOW', CLPOW)
         call hzxmld(iunit,'BTCLM',BTCLM)
         call hzxmld(iunit,'QCDLAM',QCDLAM)
         call hzxmld(iunit,'QSPAC',QSPAC)
         call hzxmld(iunit,'PTRMS',PTRMS)
         call hzxmld(iunit,'VPCUT',VPCUT)
         call hzxmld(iunit,'VQCUT',VQCUT)
         call hzxmld(iunit,'VGCUT',VGCUT)
         call hzxmld(iunit,'PTMIN',PTMIN)
         call hzxmld(iunit,'PTJIM',PTJIM)
         call hzxmld(iunit,'PRSOF',PRSOF)

         call hzxmld(iunit,'PSPLT1',PSPLT(1))
         call hzxmld(iunit,'PSPLT2',PSPLT(2))

         call hzxmld(iunit,'PHAD',PHAD)
         call hzxmld(iunit,'PRRAD',JMRAD(73))
         call hzxmld(iunit,'PHRAD',JMRAD(59))

         call hzxmli(iunit,'IPROC',IPROC)
         call hzxmli(iunit,'JIMMY',MSFLAG)
         call hzxmli(iunit,'JMUEO',JMUEO)
         call hzxmli(iunit,'ISPAC',ISPAC)
         call hzxmli(iunit,'IFLMAX',IFLMAX)
         call hzxmli(iunit,'IOPREM',IOPREM)
         call hzxmli(iunit,'NFLAV',NFLAV)

         write (iunit,*) '   </hep:generatorSettings>'
         write (iunit,*) '  </hep:run>'

      endif

      RETURN
      END
