      SUBROUTINE HZCARDS(IFLAG)
c     -------------------------
c     Read control cards into the hzcontrol common block.
c
c     Generator-independent control cards.
c
c     Call once with IFLAG=1 before generator-specific HZ*CARDS,
c     then once with IFLAG=2 afterwards.
c
c     -------------------------
      IMPLICIT NONE

c     Flag to indicate which steps to do
      INTEGER IFLAG

c  Variable and common blocks for steering parameters
#include "hzcontrol.inc"

c     Probably a bit dodgy initializing COMMON variables in DATA statements
c     within a subroutine, but at least it does seem to be done only once.

c     Initialize general parameters (unnecessary because set below ???)
      DATA hzlumi /-999./

      DATA hzxmlout /0/

      DATA hzaidaout /0/

c --- Common for FFREAD
      INTEGER nspace
      REAL space
      PARAMETER (nspace=10000)
      COMMON /CFREAD/ space(nspace)

c     Parameters for reading steering file
      INTEGER ipunit,lenocc
      PARAMETER(ipunit=5)
      CHARACTER*80 stem

c     Name of executable (full path)
      CHARACTER*255 name

      INTEGER loop_1, loop_2, itmp

      IF (IFLAG.EQ.1) THEN

c       Initialize steering parameters to defaults
        HZNEV = 1000
        HZLUMI = -1.0
        hzptmin = -999.
        hzrnd1 = 0
        hzrnd2 = 0
        hztest = .false.
        DO 23 itmp = 1, 200
          hzroutine(itmp) = .false.
          hzstab(itmp) = -9999
 23     CONTINUE

c       Need to initialize beam particle names to strings of spaces.
c       Otherwise FFREAD only sets characters in blocks of 4, making string
c       comparisons fail.
        hzb1 = '        '
        hzb2 = '        '

c       Name of executable is "0th" command-line argument
        CALL GETARG(0,name)
c       File stem is first command-line argument
        CALL GETARG(1,stem)

        PRINT*,'*******************************************************'
        PRINT*,'*'
        PRINT*,'* Executable is ',name(1:LENOCC(name))
        PRINT*,'* Reading cards from ',stem(1:LENOCC(stem))//'.input'
        PRINT*,'*'
        PRINT*,'*******************************************************'
        PRINT*,''

c       Open control file for reading
        OPEN(UNIT=ipunit,FILE=stem(1:LENOCC(stem))//'.input',status
     &       ='UNKNOWN')

c ---   Initialize FFREAD
        CALL FFINIT(nspace)    ! ffread space
        CALL FFSET('SIZE',8)   ! ffread significant characters (may need more!)

c       HepML output
        CALL FFKEY('XML',hzxmlout,1,'INTE')

c       AIDA output
        CALL FFKEY('AIDA',hzaidaout,1,'INTE')

c       Beam types
        CALL FFKEY('BEAM1',hzb1,8,'TEXT')
        CALL FFKEY('BEAM2',hzb2,8,'TEXT')

c       Beam energies
        CALL FFKEY('PBEAM1',hzpb1,1,'REAL')
        CALL FFKEY('PBEAM2',hzpb2,1,'REAL')

c       Parton distribution functions for proton and photon (not used yet ???)
        CALL FFKEY('PDFPR',hzpdfpr,1,'INTE')
        CALL FFKEY('PDFPH',hzpdfph,1,'INTE')

c       Intrinsic KT in proton and photon (not used yet ???)
        CALL FFKEY('PTRMSPR',hzptrmspr,1,'REAL')
        CALL FFKEY('PTRMSPH',hzptrmsph,1,'REAL')

c       Minimum PT for hard subprocess
        CALL FFKEY('PTMIN',hzptmin,1,'REAL')

c       No. of events
        CALL FFKEY('NEV',hznev,1,'integer')

c       Luminosity
        CALL FFKEY('LUMI',hzlumi,1,'real')

c       Stable final-state particles
        CALL FFKEY('STABLE',HZSTAB,200,'integer')

c       random numbers (pythia only uses the first one).
        CALL FFKEY('RANDNUM1',hzrnd1,1,'integer')
        CALL FFKEY('RANDNUM2',hzrnd2,1,'integer')

c       Now select the analysis routines to run
        CALL FFKEY('ROUTINE',hzroutine,200,'logical')
        CALL FFKEY('TEST',hztest,1,'logical')

      ELSE IF (IFLAG.EQ.2) THEN

c ---   get the ffread cards from standard input -----------------------
        CALL FFGO

        DO 24 itmp = 1, 200
          if (hzroutine(itmp)) then
            WRITE (6,*) '*** Routine selected: ',itmp
          endif
 24     CONTINUE

      ELSE
        WRITE (6,*) 'Error in HZCARDS: IFLAG should be 1 or 2'
      ENDIF

      RETURN
      END





